#include <maps/libs/cmdline/include/cmdline.h>
#include <maps/libs/common/include/exception.h>
#include <maps/libs/log8/include/log8.h>
#include <maps/wikimap/mapspro/services/mrc/libs/common/include/pg_locks.h>
#include <maps/wikimap/mapspro/services/mrc/libs/config/include/config.h>
#include <maps/wikimap/mapspro/services/mrc/long_tasks/red_card/lib/db.h>
#include <yandex/maps/pgpool3utils/pg_advisory_mutex.h>

namespace red_card = maps::mrc::red_card;

int main(int argc, char* argv[])
try {
    auto parser = maps::cmdline::Parser{};
    auto syslog = parser.string("syslog-tag")
                      .help("redirect log output to syslog with given tag");
    auto configPath = parser.string("config").help("path to configuration");
    auto secretVersion = parser.string("secret").help(
        "version for secrets from yav.yandex-team.ru");
    auto dryRun = parser.flag("dry-run").help("do not publish data");
    parser.parse(argc, argv);

    if (syslog.defined()) {
        maps::log8::setBackend(maps::log8::toSyslog(syslog));
    }
    auto config =
        maps::mrc::common::templateConfigFromCmdPath(secretVersion, configPath);
    auto postgres = config.makePoolHolder(maps::mrc::common::LONG_READ_DB_ID,
                                          maps::mrc::common::LONG_READ_POOL_ID);
    auto mutex = maps::pgp3utils::PgAdvisoryXactMutex{
        postgres.pool(),
        static_cast<int64_t>(maps::mrc::common::LockId::RedCard)};
    if (!dryRun && !mutex.try_lock()) {
        INFO() << "another process is ongoing";
        return EXIT_SUCCESS;
    }

    auto now = maps::chrono::TimePoint::clock::now();
    INFO() << "updating disqualified sources, dryRun=" << dryRun;
    red_card::updateDisqualifiedSources(postgres.pool(), now, dryRun);
    if (!dryRun) {
        INFO() << "updating metadata";
        red_card::updateLastRunTime(postgres.pool(), now);
    }

    INFO() << "done";
    return EXIT_SUCCESS;
}
catch (const maps::Exception& e) {
    ERROR() << e;
    return EXIT_FAILURE;
}
catch (const std::exception& e) {
    ERROR() << e.what();
    return EXIT_FAILURE;
}
